# for localized messages  	 
from . import _

import os
import enigma
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigYesNo, ConfigSelection
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Components.ConfigList import ConfigListScreen
from Components.ActionMap import ActionMap
from Components.Button import Button
from Components.Label import Label
from Plugins.Plugin import PluginDescriptor
from Components.Pixmap import Pixmap, MultiPixmap

# Configuration
config.plugins.hddsleep = ConfigSubsection()
config.plugins.hddsleep.enabled = ConfigYesNo(default = False)
config.plugins.hddsleep.sda = ConfigSelection(default = "0", choices = [("0","none"),("30","30 seconds"),("60","1 minute"),("180","3 minutes"),("300","5 minutes"),("600","10 minutes"),("1800","30 minutes"),("3600","60 minutes")])
config.plugins.hddsleep.sdb = ConfigSelection(default = "0", choices = [("0","none"),("30","30 seconds"),("60","1 minute"),("180","3 minutes"),("300","5 minutes"),("600","10 minutes"),("1800","30 minutes"),("3600","60 minutes")])
config.plugins.hddsleep.sdc = ConfigSelection(default = "0", choices = [("0","none"),("30","30 seconds"),("60","1 minute"),("180","3 minutes"),("300","5 minutes"),("600","10 minutes"),("1800","30 minutes"),("3600","60 minutes")])
config.plugins.hddsleep.sdd = ConfigSelection(default = "0", choices = [("0","none"),("30","30 seconds"),("60","1 minute"),("180","3 minutes"),("300","5 minutes"),("600","10 minutes"),("1800","30 minutes"),("3600","60 minutes")])
config.plugins.hddsleep.sde = ConfigSelection(default = "0", choices = [("0","none"),("30","30 seconds"),("60","1 minute"),("180","3 minutes"),("300","5 minutes"),("600","10 minutes"),("1800","30 minutes"),("3600","60 minutes")])
config.plugins.hddsleep.logfile = ConfigYesNo(default = False)

# PATH variables
SCRIPT 	= "${libdir}/enigma2/python/Plugins/Extensions/HddSleep/hddsleep.sh"
DEFAULT = "/etc/enigma2/hddsleep"
LINK 	= "/etc/rcS.d/S99hddsleep"
TEMP 	= "/tmp/hddsleep.tmp"
LOG 	= "/tmp/hddsleep.log"

# Global
version = "1.143"
cfg = config.plugins.hddsleep

#items in menu
#       0      1      2      3      4      5     6       7
dev = [ None, 'sda', 'sdb', 'sdc', 'sdd', 'sde', None , 'path']
first = 1
last = 6 #( 5 + 1 )
path = 7

SET_AUTORUN = "ln -s %s %s" % (SCRIPT,LINK)
#RM_AUTORUN  = "rm %s" % (LINK)


##################################
# Configuration GUI
class Config(ConfigListScreen,Screen):
	skin = """
<screen position="center,center" size="560,400" title="HddSleep Configuration" >
	<ePixmap name="red"    position="0,0"   zPosition="2" size="140,40" pixmap="skin_default/buttons/red.png" transparent="1" alphatest="on" />
	<ePixmap name="green"  position="140,0" zPosition="2" size="140,40" pixmap="skin_default/buttons/green.png" transparent="1" alphatest="on" />
	<ePixmap name="yellow" position="280,0" zPosition="2" size="140,40" pixmap="skin_default/buttons/yellow.png" transparent="1" alphatest="on" /> 
	<ePixmap name="blue"   position="420,0" zPosition="2" size="140,40" pixmap="skin_default/buttons/blue.png" transparent="1" alphatest="on" /> 

	<widget name="key_red" position="0,0" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" /> 
	<widget name="key_green" position="140,0" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" /> 
	<widget name="key_yellow" position="280,0" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />
	<widget name="key_blue" position="420,0" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />

	<widget name="config" position="30,40" size="520,200" scrollbarMode="showOnDemand"/>

	<ePixmap pixmap="skin_default/div-h.png" position="0,250" zPosition="1" size="560,2" />

	<widget name="devices" position="10,265" size="200,25" font="Regular;20" />
	<widget name="mounted" position="240,265" size="310,120" font="Regular;20" />

	<widget name="process" position="10,340" size="90,25" font="Regular;20" />
	<widget name="daemon0" alphatest="on" pixmap="skin_default/buttons/button_green_off.png" position="100,344" size="15,16" zPosition="10" transparent="1"/>
	<widget name="daemon1" alphatest="on" pixmap="skin_default/buttons/button_green.png" position="100,344" size="15,16" zPosition="10" transparent="1"/>

	<ePixmap pixmap="skin_default/div-h.png" position="0,376" zPosition="1" size="560,2" />
	<ePixmap alphatest="on" pixmap="skin_default/icons/clock.png" position="480,383" size="14,14" zPosition="3"/>
	<widget font="Regular;18" halign="left" position="505,380" render="Label" size="55,20" source="global.CurrentTime" transparent="1" valign="center" zPosition="3">
		<convert type="ClockToText">Default</convert>
	</widget>
	<widget name="statusbar" position="10,380" size="470,20" font="Regular;18" />
	
	<widget name="0" pixmaps="skin_default/buttons/button_green_off.png,skin_default/buttons/button_green.png" position="10,43" zPosition="10" size="15,16" transparent="1" alphatest="on"/>
	<widget name="1" pixmaps="skin_default/buttons/button_blue_off.png,skin_default/buttons/button_blue.png" position="10,68" zPosition="10" size="15,16" transparent="1" alphatest="on"/>
	<widget name="2" pixmaps="skin_default/buttons/button_blue_off.png,skin_default/buttons/button_blue.png" position="10,93" zPosition="10" size="15,16" transparent="1" alphatest="on"/>
	<widget name="3" pixmaps="skin_default/buttons/button_blue_off.png,skin_default/buttons/button_blue.png" position="10,118" zPosition="10" size="15,16" transparent="1" alphatest="on"/>
	<widget name="4" pixmaps="skin_default/buttons/button_blue_off.png,skin_default/buttons/button_blue.png" position="10,143" zPosition="10" size="15,16" transparent="1" alphatest="on"/>
	<widget name="5" pixmaps="skin_default/buttons/button_blue_off.png,skin_default/buttons/button_blue.png" position="10,168" zPosition="10" size="15,16" transparent="1" alphatest="on"/>
	<widget name="6" pixmaps="skin_default/buttons/button_green_off.png,skin_default/buttons/button_green.png" position="10,193" zPosition="10" size="15,16" transparent="1" alphatest="on"/>
</screen>"""


	def __init__(self, session, args = 0):
		self.skin = Config.skin
		self.session = session
		self.setup_title = _("HddSleep Configuration")
		Screen.__init__(self, session)

		configList = [
			getConfigListEntry(_("Enable sleeping"), cfg.enabled),
			getConfigListEntry("%s %s" % (_("Sleep time for"),dev[1]), cfg.sda),
			getConfigListEntry("%s %s" % (_("Sleep time for"),dev[2]), cfg.sdb),
			getConfigListEntry("%s %s" % (_("Sleep time for"),dev[3]), cfg.sdc),
			getConfigListEntry("%s %s" % (_("Sleep time for"),dev[4]), cfg.sdd),
			getConfigListEntry("%s %s" % (_("Sleep time for"),dev[5]), cfg.sde),
			getConfigListEntry(_("Create log file"), cfg.logfile),
			getConfigListEntry(_("Path for epg.dat"), config.misc.epgcache_filename),
			]
		ConfigListScreen.__init__(self, configList, session=self.session, on_change = self.changedEntry)

		self["key_red"] = Button(_("Cancel"))
		self["key_green"] = Button(_("Ok"))
		self["key_yellow"] = Button()
		self["key_blue"] = Button()
		self["devices"] = Label(_("Mounted devices:"))
		self["mounted"] = Label()
		self["process"] = Label(_("Process:"))
		self["statusbar"] = Label()
		self["daemon0"] = Pixmap()
		self["daemon0"].hide()
		self["daemon1"] = Pixmap()
		self["daemon1"].hide()

		for i in range(last+1): # 0-6 step1 ... < 7
			self["%s" % (i)] = MultiPixmap()

		self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
		{
			"red": self.cancel,
			"green": self.save,
			"blue": self.sleepNow,
			"yellow": self.setEPG,
			"cancel": self.cancel,
			"ok": self.save,
		}, -2)
		self.cfg_changes = False
		self.onChangedEntry = []
		self.mnt = {}
		self.idx = 0
		self["config"].onSelectionChanged.append(self.configPosition)
		self.info()

	def changedEntry(self):
		self.menuIcons()
		for x in self.onChangedEntry:
			x()
	
	def configPosition(self):
		self.menuIcons()
		self.getPosition()
		self.menuButton()

	def menuButton(self):
		self["key_blue"].setText("")
		self["key_yellow"].setText("")
		if self.idx in range (first,last): # from 1 with step 1 < 6
			if self.isMounted(self.idx):
				self["key_blue"].setText(_("Sleep") + " %s" % (dev[self.idx]))
				self["key_yellow"].setText(_("Epg.dat here"))
		elif self.idx == path:		
			self["key_yellow"].setText(_("Default path"))

	def setEPG(self):
		if self.idx in range(first,last):
			if self.mnt.has_key(dev[self.idx]):
				config.misc.epgcache_filename.value = self.mnt[dev[self.idx]]+"/epg.dat"
				self["config"].invalidate(self["config"].list[path])
				self.epgMsg()
		elif self.idx == path:
			config.misc.epgcache_filename.value = config.misc.epgcache_filename.default
			self["config"].invalidate(self["config"].list[path])
			self.epgMsg()

	def epgMsg(self):
		if config.misc.epgcache_filename.value != config.misc.epgcache_filename.default:
			self.session.open(MessageBox, _("Before uninstall this plugin set epg.dat path back to default!"), MessageBox.TYPE_INFO, timeout=6)

	def sleepNow(self):
		if self.idx in range(first,last):
			if self.mnt.has_key(dev[self.idx]):
				os.system("hd-idle -t %s" % (dev[self.idx]))

	def getPosition(self):
		for self.idx, x in enumerate(self["config"].list):
			if x[1] == self["config"].getCurrent()[1]:
				return
		self.idx = -1 

	def menuIcons(self):
		for i in range(first,last): # same as in (1,2,3,4,5)
			if self["config"].list[i][1].getValue() == "0":
				self["%s" % (i)].setPixmapNum(0)
			else:
				self["%s" % (i)].setPixmapNum(1)
		for i in range(0,last+1,last): #from 0 with step 6  < 7
			if self["config"].list[i][1].getValue() == False:
				self["%s" % (i)].setPixmapNum(0)
			else:
				self["%s" % (i)].setPixmapNum(1)

	def getMount(self):
		os.system("mount | sort > %s" % (TEMP))
		tmp=""
		try:		
			fd = open(TEMP, "r")
			if fd:
				for line in fd.readlines():
					if line.find('/dev/sd') == 0:
						self.getDisks(line)
						p = line.find('type ')
						tmp += line[:p-1] + "\n"
				self["mounted"].setText("%s" % (tmp))
				fd.close()
				os.system("rm %s" % (TEMP))
	
		except Exception, e:
			print "[hddsleep] getMount FAIL:", e

	def getDisks(self,d):
		self.mnt[d[d.find('sd'):d.find(' ')-1]] = d[d.find('on ')+2:d.find('type ')-1]

	def hideIcon(self):
		for i in range(first,last):
			if not self.mnt.has_key(dev[i]):
				self["%i" % (i)].hide()

	def isMounted(self, i):
		if not self.mnt.has_key(dev[i]):
			return 0
		return 1

	def autoRun(self): 
		os.system(SET_AUTORUN)
		print "[hddsleep] created link for autorun"

	def isRunning(self):
		os.system("pidof hd-idle > %s" % (TEMP))
		try:
			fd = open(TEMP,"r")
			if fd:
				test = fd.readline()
				fd.close()
				os.system("rm %s" % (TEMP))
				if test == "\n" or test == "":
					return 0
				else:
					return 1
		except Exception, e:
			print "[hddsleep] isRunning FAIL:", e
			return 0

	def runDaemon(self):
		self.setParams()
		cmd = SCRIPT
		if self.isRunning():
			cmd +=  " restart"
		else:
			cmd +=  " start"
		return cmd

	def run(self):
		print "[hddsleep] start of hd-idle"
		os.system(self.runDaemon())

	def isAutorun(self): 
		try:
			fd = open(LINK,"r")
			fd.close()
			return 1
		except Exception, e:
			return 0

	def info(self):
		if not self.isAutorun():
			self["statusbar"].setText(_("Autorun is disabled!"))
		else:
			self["statusbar"].setText(_("Version")+ " %s" % (version))
		if self.isRunning():
			self["daemon0"].hide()
			self["daemon1"].show()
		else:
			self["daemon1"].hide()
			self["daemon0"].show()
		self.getMount()
		self.hideIcon()

	def stopDaemon(self):
		cmd = SCRIPT + " stop"
		return cmd

	def stop(self):
		print "[hddsleep] canceling of hd-idle"
		os.system(self.stopDaemon())

	def save(self):
		self.setParams()
		if not self.isAutorun():
			self.autoRun()
		if cfg.enabled.value:
			self.run()
		else:
			self.stop()
		self.keySave()

	def cancel(self):
		self.keyCancel()

	# Parameters
	def setParams(self):
		enabled = "true"
		if not cfg.enabled.value:
			enabled = "false"
		line_1 = "START_HD_IDLE=%s" % (enabled)

#		line  = "HD_IDLE_OPTS="
		line_2 = "HD_IDLE_OPTS="
		line_2 += "'\"'"
		if cfg.sda.value != "0":
			line_2 += " -a sda -i %s" % (cfg.sda.value)
		if cfg.sdb.value != "0":
			line_2 += " -a sdb -i %s" % (cfg.sdb.value)
		if cfg.sdc.value != "0":
			line_2 += " -a sdc -i %s" % (cfg.sdc.value)
		if cfg.sdd.value != "0":
			line_2 += " -a sdd -i %s" % (cfg.sdd.value)
		if cfg.sde.value != "0":
			line_2 += " -a sde -i %s" % (cfg.sde.value)
		if cfg.logfile.value:
			line_2 += " -l %s" % (LOG)
		line_2 += "'\"'"
		print line_2
		try:
			#so much slow
#			file = open(DEFAULT, "w")
#			file.write("%s\n%s\"%s\"" % (line_1,line,line_2))
#			file.close()
			os.system("echo %s >  %s" % (line_1,DEFAULT))
			os.system("echo %s >> %s" % (line_2,DEFAULT))
			os.system("chmod 644 %s" % (DEFAULT))

		except Exception, e:
			print "[hddsleep] setParam FAIL:", e

def HddSleepMain(session, **kwargs):
	session.open(Config)

##################################

def Plugins(**kwargs):
	result = [PluginDescriptor(name="HddSleep",description = _("HDD sleeptime settings"),where = PluginDescriptor.WHERE_PLUGINMENU,icon = 'plugin.png',fnc = HddSleepMain)]
	return result
